@@ -48,10 +48,9 @@ NET_DIRS="wireless mac80211 rfkill"
 CODE_METRICS=code-metrics.txt
 
 usage() {
-	printf "Usage: $0 [ refresh] [ --help | -h | -s | -n | -p | -c ]\n"
+	printf "Usage: $0 [ refresh] [ --help | -h | -n | -p | -c ]\n"
 
 	printf "${GREEN}%10s${NORMAL} - will update your all your patch offsets using quilt\n" "refresh"
-	printf "${GREEN}%10s${NORMAL} - get and apply pending-stable/ fixes purging old files there\n" "-s"
 	printf "${GREEN}%10s${NORMAL} - apply the patches linux-next-cherry-picks directory\n" "-n"
 	printf "${GREEN}%10s${NORMAL} - apply the patches on the linux-next-pending directory\n" "-p"
 	printf "${GREEN}%10s${NORMAL} - apply the patches on the crap directory\n" "-c"
@@ -67,43 +66,36 @@ brag_backport() {
 	printf "${GREEN}%10s${NORMAL} - backport from compat module\n" $COMPAT_FILES_CODE
 	printf "${GREEN}%10s${NORMAL} - total backport code\n" $COMPAT_ALL_CHANGES
 	printf "${RED}%10s${NORMAL} - %% of code consists of backport work\n" \
-		$(perl -e 'printf("%.4f", 100 * '$COMPAT_ALL_CHANGES' / '$1');')
-}
-
-nag_pending_stable() {
-	printf "${YELLOW}%10s${NORMAL} - Code changes brought in from pending-stable\n" $2
-	printf "${YELLOW}%10s${NORMAL} - Code additions brought in from pending-stable\n" $3
-	printf "${YELLOW}%10s${NORMAL} - Code deletions brought in from pending-stable\n" $4
-	printf "${RED}%10s${NORMAL} - %% of code being cherry picked from pending-stable\n" $(perl -e 'printf("%.4f", 100 * '$2' / '$1');')
+		$(perl -e 'printf("%.2f", 100 * '$COMPAT_ALL_CHANGES' / '$1');')
 }
 
 nag_next_cherry_pick() {
 	printf "${YELLOW}%10s${NORMAL} - Code changes brought in from linux-next\n" $2
 	printf "${YELLOW}%10s${NORMAL} - Code additions brought in from linux-next\n" $3
 	printf "${YELLOW}%10s${NORMAL} - Code deletions brought in from linux-next\n" $4
-	printf "${RED}%10s${NORMAL} - %% of code being cherry picked from linux-next\n" $(perl -e 'printf("%.4f", 100 * '$2' / '$1');')
+	printf "${RED}%10s${NORMAL} - %% of code being cherry picked from linux-next\n" $(perl -e 'printf("%.2f", 100 * '$2' / '$1');')
 }
 
 nag_pending() {
 	printf "${YELLOW}%10s${NORMAL} - Code changes posted but not yet merged\n" $2
 	printf "${YELLOW}%10s${NORMAL} - Code additions posted but not yet merged\n" $3
 	printf "${YELLOW}%10s${NORMAL} - Code deletions posted but not yet merged\n" $4
-	printf "${RED}%10s${NORMAL} - %% of code not yet merged\n" $(perl -e 'printf("%.4f", 100 * '$2' / '$1');')
+	printf "${RED}%10s${NORMAL} - %% of code not yet merged\n" $(perl -e 'printf("%.2f", 100 * '$2' / '$1');')
 }
 
 nag_crap() {
 	printf "${RED}%10s${NORMAL} - Crap changes not yet posted\n" $2
-	printf "${RED}%10s${NORMAL} - Crap additions not yet posted\n" $3
+	printf "${RED}%10s${NORMAL} - Crap additions not yet merged\n" $3
 	printf "${RED}%10s${NORMAL} - Crap deletions not yet posted\n" $4
-	printf "${RED}%10s${NORMAL} - %% of crap code\n" $(perl -e 'printf("%.4f", 100 * '$2' / '$1');')
+	printf "${RED}%10s${NORMAL} - %% of crap code\n" $(perl -e 'printf("%.2f", 100 * '$2' / '$1');')
 }
 
 nagometer() {
 	CHANGES=0
 
 	ORIG_CODE=$2
-	ADD=$(grep -Hc ^+ $1/*.patch| awk -F":" 'BEGIN {sum=0} {sum += $2} END { print sum}')
-	DEL=$(grep -Hc ^- $1/*.patch| awk -F":" 'BEGIN {sum=0} {sum += $2} END { print sum}')
+	ADD=$(grep -c ^+ $1/*.patch| awk -F":" 'BEGIN {sum=0} {sum += $2} END { print sum}')
+	DEL=$(grep -c ^- $1/*.patch| awk -F":" 'BEGIN {sum=0} {sum += $2} END { print sum}')
 	# Total code is irrelevant unless you take into account each part,
 	# easier to just compare against the original code.
 	# let TOTAL_CODE=$ORIG_CODE+$ADD-$DEL
@@ -114,9 +106,6 @@ nagometer() {
 	"patches")
 		brag_backport $ORIG_CODE $CHANGES $ADD $DEL
 		;;
-	"pending-stable")
-		nag_pending_stable $ORIG_CODE $CHANGES $ADD $DEL
-		;;
 	"linux-next-cherry-picks")
 		nag_next_cherry_pick $ORIG_CODE $CHANGES $ADD $DEL
 		;;
@@ -134,8 +123,6 @@ nagometer() {
 
 EXTRA_PATCHES="patches"
 REFRESH="n"
-GET_STABLE_PENDING="n"
-POSTFIX_RELEASE_TAG=""
 if [ $# -ge 1 ]; then
 	if [ $# -gt 4 ]; then
 		usage $0
@@ -146,25 +133,16 @@ if [ $# -ge 1 ]; then
 		exit
 	fi
 	while [ $# -ne 0 ]; do
-		if [[ "$1" = "-s" ]]; then
-			GET_STABLE_PENDING="y"
-			EXTRA_PATCHES="${EXTRA_PATCHES} pending-stable" 
-			POSTFIX_RELEASE_TAG="${POSTFIX_RELEASE_TAG}s"
-			shift; continue;
-		fi
 		if [[ "$1" = "-n" ]]; then
 			EXTRA_PATCHES="${EXTRA_PATCHES} linux-next-cherry-picks"
-			POSTFIX_RELEASE_TAG="${POSTFIX_RELEASE_TAG}n"
 			shift; continue;
 		fi
 		if [[ "$1" = "-p" ]]; then
 			EXTRA_PATCHES="${EXTRA_PATCHES} linux-next-pending"
-			POSTFIX_RELEASE_TAG="${POSTFIX_RELEASE_TAG}p"
 			shift; continue;
 		fi
 		if [[ "$1" = "-c" ]]; then
 			EXTRA_PATCHES="${EXTRA_PATCHES} crap"
-			POSTFIX_RELEASE_TAG="${POSTFIX_RELEASE_TAG}c"
 			shift; continue;
 		fi
 		if [[ "$1" = "refresh" ]]; then
@@ -253,7 +231,6 @@ mkdir -p include/linux/ include/net/ include/linux/usb \
 	include/linux/unaligned \
 	include/linux/spi \
 	include/trace \
-	include/pcmcia \
 	net/mac80211/ net/wireless/ \
 	net/rfkill/ \
 	drivers/ssb/ \
@@ -376,62 +353,8 @@ cp -a $GIT_COMPAT_TREE/scripts/ $COMPAT/
 cp -a $GIT_COMPAT_TREE/include/linux/* include/linux/
 cp -a $GIT_COMPAT_TREE/include/net/* include/net/
 cp -a $GIT_COMPAT_TREE/include/trace/* include/trace/
-cp -a $GIT_COMPAT_TREE/include/pcmcia/* include/pcmcia/
 rm -f $COMPAT/*.mod.c
 
-# files we suck in for compat-wireless
-export WSTABLE="
-        net/wireless/
-        net/wireless/
-        net/mac80211/
-        net/rfkill/
-        drivers/net/wireless/
-        net/bluetooth/
-        drivers/bluetooth/
-        drivers/net/atl1c/
-        drivers/net/atl1e/
-        drivers/net/atlx/
-        include/linux/nl80211.h
-        include/linux/rfkill.h
-        include/net/cfg80211.h
-	include/net/mac80211.h
-        include/net/regulatory.h
-        include/net/cfg80211.h"
-
-# Stable pending, if -n was passed
-if [[ "$GET_STABLE_PENDING" = y ]]; then
-	LAST_DIR=$PWD
-	cd $GIT_TREE
-	if [ -f localversion* ]; then
-		echo -e "You should be using a stable tree to use the -s option"
-		exit 1
-	fi
-
-	# we now assume you are using a stable tree
-	cd $GIT_TREE
-	LAST_STABLE_UPDATE=$(git describe --abbrev=0)
-	NEXT_TREE="/home/$USER/linux-next/"
-	if [ ! -d $NEXT_TREE ]; then
-		echo -e "You are expected to have $NEXT_TREE directory when using -s"
-		exit 1
-	fi
-	cd $NEXT_TREE
-	PENDING_STABLE_DIR="pending-stable/"
-
-	rm -rf $CHERRY_PICK_DIR
-	echo -e "${GREEN}Generating stable cherry picks... ${NORMAL}"
-	git format-patch --grep="stable@kernel.org" -o $PENDING_STABLE_DIR ${LAST_STABLE_UPDATE}.. $WSTABLE
-	if [ ! -d ${LAST_DIR}/${PENDING_STABLE_DIR} ]; then
-		echo -e "Assumption that ${LAST_DIR}/${PENDING_STABLE_DIR} directory exists failed"
-		exit 1
-	fi
-	echo -e "${GREEN}Purging old stable cherry picks... ${NORMAL}"
-	rm -f ${LAST_DIR}/${PENDING_STABLE_DIR}/*.patch
-	cp ${PENDING_STABLE_DIR}/*.patch ${LAST_DIR}/${PENDING_STABLE_DIR}/
-	echo -e "${GREEN}Updated stable cherry picks, review with git diff and update hunks with ./scripts/admin-update.sh -s refresh${NORMAL}"
-	cd $LAST_DIR
-fi
-
 # Refresh patches using quilt
 patchRefresh() {
 	if [ -d patches.orig ] ; then
@@ -472,7 +395,7 @@ ORIG_CODE=$(find ./ -type f -name  \*.[ch] |
 	egrep -v "^./compat/|include/linux/compat" |
 	xargs wc -l | tail -1 | awk '{print $1}')
 printf "\n${CYAN}compat-wireless code metrics${NORMAL}\n\n" > $CODE_METRICS
-printf "${PURPLE}%10s${NORMAL} - Total upstream lines of code being pulled\n" $ORIG_CODE >> $CODE_METRICS
+printf "${PURPLE}%10s${NORMAL} - Total upstream code being pulled\n" $ORIG_CODE >> $CODE_METRICS
 
 for dir in $EXTRA_PATCHES; do
 	LAST_ELEM=$dir
@@ -512,11 +435,7 @@ echo -e "${GREEN}Updated${NORMAL} from local tree: ${BLUE}${GIT_TREE}${NORMAL}"
 echo -e "Origin remote URL: ${CYAN}${GIT_REMOTE_URL}${NORMAL}"
 cd $DIR
 if [ -d ./.git ]; then
-	if [[ ${POSTFIX_RELEASE_TAG} != "" ]]; then
-		echo -e "$(git describe)-${POSTFIX_RELEASE_TAG}" > compat_version
-	else
-		echo -e "$(git describe)" > compat_version
-	fi
+	git describe > compat_version
 
 	cd $GIT_TREE
 	TREE_NAME=${GIT_REMOTE_URL##*/}
